@@ -60,6 +60,7 @@ class App(QMainWindow):
         self.recent_files = ['']
         self.main.setFocus()
         self.setCentralWidget(self.main)
+        self.create_tool_bar()
         self.setup_gui()
         self.load_settings()
         self.show_recent_files()
@@ -86,7 +87,8 @@ class App(QMainWindow):
             #self.FONTSIZE = int(s.value("fontsize"))
             r = s.value("recent_files")
             if r != '':
-                self.recent_files = r.split(',')
+                rct = r.split(',')
+                self.recent_files = [f for f in rct if os.path.exists(f)]
 
         except Exception as e:
             print (e)
@@ -133,8 +135,44 @@ class App(QMainWindow):
         return
 
     def update_mask(self):
+        if self.mask_file == None:
+            self.masklabel.setText('')
+        else:
+            self.masklabel.setText(os.path.basename(self.mask_file))
+        return
 
-        self.masklabel.setText(self.mask_file)
+    def create_tool_bar(self):
+        """Create main toolbar"""
+
+        items = {'New project': {'action': lambda: self.new_project(ask=True),'file':'document-new'},
+                 'Open': {'action':self.load_project,'file':'document-open'},
+                 'Save': {'action': lambda: self.save_project(),'file':'save'},
+                 'Zoom out': {'action':self.zoom_out,'file':'zoom-out'},
+                 'Zoom in': {'action':self.zoom_in,'file':'zoom-in'},
+                 'Align Reads': {
+                    'action': lambda: self.run_threaded_process(self.align_files, self.alignment_completed),
+                    'file':'align-reads'},
+                 'Call Variants': {
+                    'action': lambda: self.run_threaded_process(self.variant_calling, self.processing_completed),
+                    'file':'call-variants'},
+                 'Make Phylogeny': {
+                    'action': lambda: self.run_threaded_process(self.make_phylo_tree, self.phylogeny_completed),
+                    'file':'phylogeny'},
+                 'Quit': {'action':self.quit,'file':'application-exit'}
+                }
+
+        toolbar = QToolBar("Main Toolbar")
+        self.addToolBar(toolbar)
+        for i in items:
+            if 'file' in items[i]:
+                iconfile = os.path.join(iconpath,items[i]['file']+'.png')
+                icon = QIcon(iconfile)
+            else:
+                icon = QIcon.fromTheme(items[i]['icon'])
+            btn = QAction(icon, i, self)
+            btn.triggered.connect(items[i]['action'])
+            #btn.setCheckable(True)
+            toolbar.addAction(btn)
         return
 
     def setup_gui(self):
@@ -190,13 +228,9 @@ class App(QMainWindow):
         self.right_tabs.setTabsClosable(True)
         self.right_tabs.tabCloseRequested.connect(self.close_right_tab)
         l2.addWidget(self.right_tabs)
-        self.info = widgets.Editor(right, readOnly=True)
-        font = QFont("Monospace")
-        font.setPointSize(9)
-        font.setStyleHint(QFont.TypeWriter)
-        self.info.setFont(font)
+        self.info = widgets.Editor(right, readOnly=True, fontsize=11)
         self.right_tabs.addTab(self.info, 'log')
-        self.info.setText("Welcome")
+        self.info.append("Welcome\n")
         self.m.setSizes([50,200,150])
         self.m.setStretchFactor(1,0)
 
@@ -219,6 +253,11 @@ class App(QMainWindow):
         self.statusBar.addWidget(self.progressbar, 3)
         self.progressbar.setAlignment(Qt.AlignRight)
         self.setStatusBar(self.statusBar)
+
+        #redirect stdout
+        self._stdout = StdoutRedirect()
+        self._stdout.start()
+        self._stdout.printOccur.connect(lambda x : self.info.insert(x))
         return
 
     @QtCore.Slot(int)
@@ -297,9 +336,10 @@ class App(QMainWindow):
             lambda: self.run_threaded_process(self.variant_calling, self.processing_completed))
         self.analysis_menu.addAction('Create SNP alignment',
             lambda: self.run_threaded_process(self.snp_alignment, self.snp_align_completed))
-        self.analysis_menu.addAction('Make Phylogeny', self.make_phylo_tree)
-        self.analysis_menu.addSeparator()
-        self.analysis_menu.addAction('Run Workflow', self.run)
+        self.analysis_menu.addAction('Make Phylogeny',
+            lambda: self.run_threaded_process(self.make_phylo_tree, self.phylogeny_completed))
+        #self.analysis_menu.addSeparator()
+        #self.analysis_menu.addAction('Run Workflow', self.run)
 
         self.tools_menu = QMenu('Tools', self)
         self.menuBar().addMenu(self.tools_menu)
@@ -312,22 +352,22 @@ class App(QMainWindow):
         self.tools_menu.addAction('Show Annotation', self.show_ref_annotation)
         self.tools_menu.addAction('Plot SNP Matrix', self.plot_snp_matrix)
         #self.tools_menu.addAction('Map View', self.show_map)
-        self.tools_menu.addAction('Phylogeny', self.tree_viewer)
+        self.tools_menu.addAction('Tree Viewer', self.tree_viewer)
         self.tools_menu.addSeparator()
         self.tools_menu.addAction('Check Heterozygosity', self.check_heterozygosity)
         self.tools_menu.addAction('RD Analysis (MTBC)',
             lambda: self.run_threaded_process(self.rd_analysis, self.rd_analysis_completed))
         self.tools_menu.addAction('M.bovis Spoligotyping',
             lambda: self.run_threaded_process(self.spoligotyping, self.spotyping_completed))
-        self.tools_menu.addAction('M.bovis SNP typing',
-            lambda: self.run_threaded_process(self.snp_typing, self.processing_completed))
+        #self.tools_menu.addAction('M.bovis SNP typing',
+        #    lambda: self.run_threaded_process(self.snp_typing, self.processing_completed))
 
         self.settings_menu = QMenu('&Settings', self)
         self.menuBar().addMenu(self.settings_menu)
         self.settings_menu.addAction('Set Output Folder', self.set_output_folder)
         self.settings_menu.addAction('Set Reference Sequence', self.set_reference)
         self.settings_menu.addAction('Set Annnotation (genbank)', self.set_annotation)
-        self.settings_menu.addAction('Set Filters', self.set_filters)
+        #self.settings_menu.addAction('Set Filters', self.set_filters)
         self.settings_menu.addAction('Add Mask File', self.add_mask)
         self.settings_menu.addAction('Add Sample Meta Data', self.merge_meta_data)
         self.settings_menu.addAction('Clean Up Files', self.clean_up)
@@ -348,10 +388,15 @@ class App(QMainWindow):
         for name in genomes:
             seqname = genomes[name]['sequence']
             gbfile = genomes[name]['gb']
-            def func(seqname,gbfile):
+            if 'mask' in genomes[name]:
+                mask = genomes[name]['mask']
+            else:
+                mask = None
+            def func(seqname,gbfile,mask):
                 self.set_reference(seqname)
                 self.set_annotation(gbfile)
-            receiver = lambda seqname=seqname, gb=gbfile: func(seqname, gb)
+                self.set_mask(mask)
+            receiver = lambda seqname=seqname, gb=gbfile, mask=mask: func(seqname, gb, mask)
             self.presets_menu.addAction('%s' %name, receiver)
         return
 
@@ -386,7 +431,7 @@ class App(QMainWindow):
         filename = self.proj_file
         data={}
         data['inputs'] = self.fastq_table.getDataFrame()
-        keys = ['outputdir','results','ref_genome','ref_gb']
+        keys = ['outputdir','results','ref_genome','ref_gb','mask_file']
         for k in keys:
             if hasattr(self, k):
                 data[k] = self.__dict__[k]
@@ -396,7 +441,8 @@ class App(QMainWindow):
         if hasattr(self, 'treeviewer'):
             d=self.treeviewer.saveData()
             data['treeviewer'] = d
-
+        self.opts.applyOptions()
+        data['options'] = self.opts.kwds
         self.projectlabel.setText(filename)
         pickle.dump(data, open(filename,'wb'))
         self.add_recent_file(filename)
@@ -433,12 +479,16 @@ class App(QMainWindow):
         self.proj_file = None
         self.fastq_table.setDataFrame(pd.DataFrame({'name':[]}))
         self.tabs.clear()
+        if hasattr(self, 'treeviewer'):
+            self.treeviewer.clear()
         self.ref_genome = None
         self.ref_gb = None
+        self.mask_file = None
         self.projectlabel.setText('')
         self.outdirLabel.setText(self.outputdir)
         self.clear_tabs()
         self.update_ref_genome()
+        self.update_mask()
         return
 
     def load_project(self, filename=None):
@@ -446,32 +496,37 @@ class App(QMainWindow):
 
         self.new_project()
         data = pickle.load(open(filename,'rb'))
-        keys = ['sheets','outputdir','results','ref_genome','ref_gb']
+        keys = ['sheets','outputdir','results','ref_genome','ref_gb','mask_file']
         for k in keys:
             if k in data:
                 self.__dict__[k] = data[k]
 
+        if 'options' in data:
+            self.opts.updateWidgets(data['options'])
         ft = self.fastq_table
         ft.setDataFrame(data['inputs'])
         ft.resizeColumns()
         self.update_ref_genome()
+        self.update_mask()
         self.proj_file = filename
         self.projectlabel.setText(self.proj_file)
         self.outdirLabel.setText(self.outputdir)
-        print (self.results)
+        #print (self.results)
         #if 'vcf_file' in self.results:
         #    self.show_variants()
         if 'snp_dist' in self.results:
             self.show_snpdist()
         #load any saved maps
-        if 'gisviewer' in data.keys():
-            self.show_map()
-            self.gisviewer.loadData(data['gisviewer'])
+        #if 'gisviewer' in data.keys():
+            #self.show_map()
+            #self.gisviewer.loadData(data['gisviewer'])
         #load tree view
         if 'treeviewer' in data.keys():
             self.tree_viewer()
             self.treeviewer.loadData(data['treeviewer'])
+        self.right_tabs.setCurrentIndex(0)
         self.add_recent_file(filename)
+        self.check_fastq_table()
         return
 
     def load_project_dialog(self):
@@ -534,10 +589,24 @@ class App(QMainWindow):
         bcf = os.path.join(self.outputdir, 'raw.bcf')
         if os.path.exists(bcf):
              os.remove(bcf)
+        df = self.fastq_table.model.df
+        cols = ['bam_file','mapped','total']
+        for col in cols:
+            if col in df.columns:
+                df = df.drop(columns=col)
+        self.fastq_table.model.df = df
+        self.fastq_table.refresh()
         self.results = {}
         self.treefile = None
         return
 
+    def check_fastq_table(self):
+        """Update samples file to reflect table"""
+
+        df = self.fastq_table.model.df
+        app.write_samples(df, self.outputdir)
+        return
+
     def load_fastq_table(self, filenames):
         """Append/Load fasta inputs into table"""
 
@@ -550,14 +619,16 @@ class App(QMainWindow):
         kwds = self.opts.kwds
         df = self.fastq_table.model.df
         new = app.get_samples(filenames, sep=kwds['labelsep'])
-        new['read_length'] = new.filename.apply(tools.get_fastq_info)
+        #pivoted
+        new = app.get_pivoted_samples(new)
+        new['read_length'] = new.filename1.apply(tools.get_fastq_info)
 
         if len(df)>0:
             new = pd.concat([df,new],sort=False).reset_index(drop=True)
-            new = new.drop_duplicates('filename')
+            new = new.drop_duplicates('filename1')
         self.fastq_table.setDataFrame(new)
         self.fastq_table.resizeColumns()
-        app.write_samples(df, self.outputdir)
+        app.write_samples(df[['sample']], self.outputdir)
         return
 
     def load_fastq_folder_dialog(self):
@@ -608,26 +679,16 @@ class App(QMainWindow):
         self.update_ref_genome()
         return
 
-    def set_filters(self):
-        """Set up variant filters"""
+    def set_mask(self, filename):
 
-        opts = {'filter':{'type':'textarea','default':app.default_filter,'width':300},
-                'mask filter': {'type':'checkbox','default':0},
-                'proximity filter': {'type':'checkbox','default':1},
-                }
-        dlg = widgets.MultipleInputDialog(self, opts, title='Variant filters',
-                            width=350,height=150)
-        dlg.exec_()
-        if not dlg.accepted:
-            return
-        kwds = dlg.values
+        self.mask_file = filename
+        self.update_mask()
 
-        return
+    def add_mask(self, filename):
+        """Add mask bed file"""
 
-    def add_mask(self, filename=None):
-
-        msg = "This will add a mask file"
-        reply = QMessageBox.question(self, 'Warning!', msg,
+        msg = "This will add a bed file as a mask. See help for format. Continue?"
+        reply = QMessageBox.question(self, 'Add bed file', msg,
                                         QMessageBox.No | QMessageBox.Yes )
         if reply == QMessageBox.No:
             return
@@ -682,17 +743,19 @@ class App(QMainWindow):
             if reply == QMessageBox.Cancel:
                 return
             elif reply == QMessageBox.Yes:
-                self.fastq_table.model.df = pd.read_csv(results_file)
+                df = pd.read_csv(results_file)
+                self.fastq_table.model.df = df
                 self.fastq_table.refresh()
                 self.results['vcf_file'] = os.path.join(self.outputdir, 'filtered.vcf.gz')
         self.outdirLabel.setText(self.outputdir)
         return
 
     def check_output_folder(self):
-        """check if we have an output dir"""
+        """Check if we have an output dir"""
 
         if self.outputdir == None:
-            self.show_info('You should set an output folder from the Settings menu')
+            self.show_info('You should set an output folder from the Settings menu!',
+                        color='red')
             return 0
         return 1
 
@@ -733,35 +796,47 @@ class App(QMainWindow):
         if retval == 0:
             return
         if self.ref_genome == None:
-            self.show_info('no reference genome!')
+            print('no reference genome!')
             return
         self.running = True
         self.opts.applyOptions()
         kwds = self.opts.kwds
         overwrite = kwds['overwrite']
         df = self.fastq_table.model.df
-        #rows = self.fastq_table.getSelectedRows()
-        #df = df.loc[rows]
-        msg = 'Aligning reads..\nThis may take some time.'
-        progress_callback.emit(msg)
+        rows = self.fastq_table.getSelectedRows()
+        new = df.loc[rows]
+        #print (new)
+        print ('Aligning reads. This may take some time.')
         ref = self.ref_genome
         if kwds['aligner'] == 'bwa':
             aligners.build_bwa_index(self.ref_genome)
         elif kwds['aligner'] == 'subread':
             aligners.build_subread_index(self.ref_genome)
 
-        progress_callback.emit('Using reference genome: %s' %ref)
+        print('Using reference genome: %s' %ref)
         path = os.path.join(self.outputdir, 'mapped')
         if not os.path.exists(path):
             os.makedirs(path)
-        samples = app.align_reads(df, idx=ref, outdir=path, overwrite=overwrite, threads=kwds['threads'],
+        new = app.align_reads(new, idx=ref, outdir=path, overwrite=overwrite,
+                        threads=kwds['threads'],
                         aligner=kwds['aligner'],
+                        platform=kwds['platform'],
                         callback=progress_callback.emit)
-        samples.to_csv(os.path.join(self.outputdir,'samples.csv'),index=False)
-        summ = app.results_summary(samples)
-        summ.to_csv(os.path.join(self.outputdir,'summary.csv'),index=False)
+        self.update_table(new)
+        #df.to_csv(os.path.join(self.outputdir,'samples.csv'),index=False)
+        #summ = app.results_summary(samples)
+        #summ.to_csv(os.path.join(self.outputdir,'summary.csv'),index=False)
         #rewrite samples in case check_missing
-        app.write_samples(samples, self.outputdir)
+        #app.write_samples(df, self.outputdir)
+        return
+
+    def update_table(self, new):
+        """Update table with changed rows"""
+
+        df = self.fastq_table.model.df
+        print (new)
+        cols = new.columns
+        df.loc[df['sample'].isin(new['sample']), cols] = new[cols]
         return
 
     def variant_calling(self, progress_callback=None):
@@ -776,18 +851,20 @@ class App(QMainWindow):
         overwrite = kwds['overwrite']
         threads = int(kwds['threads'])
         filters = kwds['filters']
+        proximity = kwds['proximity']
         df = self.fastq_table.model.df
         path = self.outputdir
 
         gff_file = os.path.join(path, self.ref_gb+'.gff')
         tools.gff_bcftools_format(self.ref_gb, gff_file)
-        #use trimmed files if present in table
-        bam_files = list(df.bam_file.unique())
 
+        bam_files = list(df.bam_file.dropna().unique())
+        print ('Calling with %s of %s samples aligned' %(len(bam_files), len(df)))
         self.results['vcf_file'] = app.variant_calling(bam_files, self.ref_genome, path,
                                     threads=threads, relabel=True,
                                     overwrite=overwrite, filters=filters,
-                                    gff_file=gff_file,
+                                    gff_file=gff_file, mask=self.mask_file,
+                                    custom_filters=proximity,
                                     callback=progress_callback.emit)
         self.results['nuc_matrix'] = os.path.join(self.outputdir, 'core.txt')
         self.results['csq_matrix'] = os.path.join(self.outputdir, 'csq.matrix')
@@ -829,7 +906,7 @@ class App(QMainWindow):
         self.opts.applyOptions()
         kwds = self.opts.kwds
         vcf_file = self.results['vcf_file']
-        progress_callback.emit('Making SNP alignment')
+        print('Making SNP alignment')
         result, smat = tools.fasta_alignment_from_vcf(vcf_file,
                                                 callback=progress_callback.emit)
         #print (result)
@@ -849,7 +926,8 @@ class App(QMainWindow):
         self.show_snpdist()
         return
 
-    def make_phylo_tree(self, method='fasttree'):
+    def make_phylo_tree(self, progress_callback=None, method='raxml'):
+        """Make phylogenetic tree"""
 
         corefasta = os.path.join(self.outputdir, 'core.fa')
         bootstraps = 100
@@ -860,11 +938,16 @@ class App(QMainWindow):
             outfile = os.path.join(self.outputdir,'fasttree.newick')
             treefile = trees.run_fasttree(corefasta, bootstraps=bootstraps, outpath=self.outputdir)
 
-        self.show_tree()
         self.treefile = outfile
         return
 
+    def phylogeny_completed(self):
+
+        self.processing_completed()
+        self.show_tree()
+
     def show_tree(self):
+        """Show current tree"""
 
         self.tree_viewer()
         filename = os.path.join(self.outputdir,'RAxML_bipartitions.variants')
@@ -873,6 +956,7 @@ class App(QMainWindow):
         return
 
     def tree_viewer(self):
+        """Show tree viewer"""
 
         from . import phylo
         if not hasattr(self, 'treeviewer'):
@@ -902,16 +986,16 @@ class App(QMainWindow):
     def processing_completed(self):
         """Generic process completed"""
 
-        self.info.append("finished")
         self.progressbar.setRange(0,1)
         self.running = False
         self.fastq_table.refresh()
+        print ('finished')
         return
 
     def alignment_completed(self):
         """Alignment/calling completed"""
 
-        self.info.append("finished")
+        print("finished")
         self.progressbar.setRange(0,1)
         df = self.fastq_table.getDataFrame()
         self.fastq_table.refresh()
@@ -921,7 +1005,7 @@ class App(QMainWindow):
     def run(self):
         """Run all steps"""
 
-        self.run_threaded_process(self.run_trimming, self.processing_completed)
+        #self.run_threaded_process(self.run_trimming, self.processing_completed)
         return
 
     def run_threaded_process(self, process, on_complete):
@@ -938,7 +1022,6 @@ class App(QMainWindow):
 
     def progress_fn(self, msg):
 
-        print (msg)
         self.info.append(msg)
         self.info.verticalScrollBar().setValue(1)
         return
@@ -953,27 +1036,42 @@ class App(QMainWindow):
         self.tabs.setCurrentIndex(i)
         return
 
-    def quality_summary(self, row):
-        """Summary of a single fastq file"""
+    def sample_details(self, row):
 
         df = self.fastq_table.model.df
         row = self.fastq_table.getSelectedRows()[0]
         data = df.iloc[row]
-        name = 'qual:'+data['name']
-        if name in self.get_tab_names():
-            return
-        w = widgets.PlotViewer(self)
-        fig,ax = plt.subplots(2,1, figsize=(7,5), dpi=65, facecolor=(1,1,1), edgecolor=(0,0,0))
-        axs=ax.flat
-        if not os.path.exists(data.filename):
-            self.show_info('This file is missing.')
-            return
-        tools.plot_fastq_qualities(data.filename, ax=axs[0])
-        tools.plot_fastq_gc_content(data.filename, ax=axs[1])
-        plt.tight_layout()
-        w.show_figure(fig)
-        i = self.tabs.addTab(w, name )
-        self.tabs.setCurrentIndex(i)
+        pd.set_option('display.max_colwidth', None)
+        print (data)
+        print ()
+        return
+
+    def quality_summary(self, row):
+        """Summary of a single sample, both files"""
+
+        df = self.fastq_table.model.df
+        row = self.fastq_table.getSelectedRows()[0]
+        data = df.iloc[row]
+        if 'filename2' in df.columns:
+            colnames = zip([data.name1, data.name2], [data.filename1, data.filename2])
+        else:
+            colnames = [[data.name1, data.filename1]]
+        for name,fname in colnames:
+            label = 'qual:'+name
+            if label in self.get_tab_names():
+                return
+            w = widgets.PlotViewer(self)
+            fig,ax = plt.subplots(2,1, figsize=(7,5), dpi=65, facecolor=(1,1,1), edgecolor=(0,0,0))
+            axs=ax.flat
+            if not os.path.exists(fname):
+                self.show_info('This file is missing.')
+                return
+            tools.plot_fastq_qualities(fname, ax=axs[0])
+            tools.plot_fastq_gc_content(fname, ax=axs[1])
+            plt.tight_layout()
+            w.show_figure(fig)
+            i = self.tabs.addTab(w, label )
+            self.tabs.setCurrentIndex(i)
         return
 
     def add_read_lengths(self, progress_callback):
@@ -983,7 +1081,7 @@ class App(QMainWindow):
         rows = self.fastq_table.getSelectedRows()
         data = df.iloc[rows]
         for i,r in data.iterrows():
-            df.loc[i,'reads'] = tools.get_fastq_length(r.filename)
+            df.loc[i,'reads'] = tools.get_fastq_length(r.filename1)
         return
 
     def add_mapping_stats(self, progress_callback):
@@ -999,6 +1097,9 @@ class App(QMainWindow):
             return s
 
         for i,r in data.iterrows():
+            if pd.isnull(r.bam_file):
+                print ('no bam file')
+                continue
             s = get_stats(r.bam_file)
             df.loc[i,'mapped'] = s['mapped']
             df.loc[i,'total'] = s['total']
@@ -1011,6 +1112,12 @@ class App(QMainWindow):
         df = self.fastq_table.model.df
         row = self.fastq_table.getSelectedRows()[0]
         data = df.iloc[row]
+        if 'bam_file' not in df.columns:
+            print ('No bam file info. Have reads been aligned?')
+            return
+        if pd.isnull(data.bam_file):
+            print ('no bam file')
+            return
         d = tools.samtools_flagstat(data.bam_file)
         df = pd.DataFrame(d.items())
         self.info.append(data.bam_file)
@@ -1024,7 +1131,7 @@ class App(QMainWindow):
         df = self.fastq_table.model.df
         row = self.fastq_table.getSelectedRows()[0]
         data = df.iloc[row]
-        name = 'aln:'+data['name']
+        name = 'aln:'+data['sample']
         if name in self.get_tab_names():
             return
         #text view using samtools tview
@@ -1046,7 +1153,7 @@ class App(QMainWindow):
         out = os.path.join(self.outputdir,'qc_report.pdf')
         if not os.path.exists(out):
             def func(progress_callback):
-                tools.pdf_qc_reports(df.filename, out)
+                tools.pdf_qc_reports(df.filename1, out)
                 import webbrowser
                 webbrowser.open_new(out)
             self.run_threaded_process(func, self.processing_completed)
@@ -1073,7 +1180,7 @@ class App(QMainWindow):
 
         idx = self.right_tabs.addTab(bv, 'snp dist')
         self.right_tabs.setCurrentIndex(idx)
-
+        return
 
     def show_browser_tab(self, link, name):
 
@@ -1125,42 +1232,43 @@ class App(QMainWindow):
         from . import snp_typing
         df = self.fastq_table.model.df
         #use ALL snp sites including uninformative
-        nucmat = pd.read_csv(self.results['nuc_matrix'],sep=' ',index_col=0)
+        if not 'snp_dist' in self.results:
+            print ('You need to create a SNP alignment first')
+            return
+        snpmat = pd.read_csv(self.results['snp_dist'],sep=' ',index_col=0)
         #print (nucmat)
         rows = self.fastq_table.getSelectedRows()
         data = df.iloc[rows]
         snptable = snp_typing.clade_snps
-        res = snp_typing.type_samples(nucmat)
-
+        res = snp_typing.type_samples(snpmat)
+        print (res)
         return
 
     def spoligotyping(self, progress_callback):
         """Mbovis spo typing tool"""
 
-        msg = 'This tool is designed for strain typing of M.bovis isolates. '
-        reply = QMessageBox.question(self, 'Continue?', msg,
-                                        QMessageBox.Yes | QMessageBox.No, QMessageBox.No)
-        if reply == QMessageBox.No:
-            return
-
+        self.running == True
+        self.opts.applyOptions()
+        kwds = self.opts.kwds
         df = self.fastq_table.model.df
-        rows = self.fastq_table.getSelectedRows()
-        data = df.iloc[rows]
+        data = self.get_selected()
+        if data is None or len(data) == 0:
+            return
         res=[]
+        cols = ['sample','spotype','sb']
         for i,r in data.iterrows():
             name = r['sample']
-            s = tools.get_spoligotype(r.filename, reads_limit=500000, threshold=2)
+            s = tools.get_spoligotype(r.filename1, threads=kwds['threads'])
             sb = tools.get_sb_number(s)
-            #print (name, sb)
-            res.append([name,sb])
-        print (pd.DataFrame(res))
-
+            print (name, s, sb)
+            #set new values in place
+            df.loc[i,cols] = [name,s,sb]
         return
 
     def spotyping_completed(self):
         """Typing completed"""
 
-        self.info.append("finished")
+        print("finished")
         self.progressbar.setRange(0,1)
         self.fastq_table.refresh()
         self.running = False
@@ -1169,65 +1277,83 @@ class App(QMainWindow):
     def rd_analysis(self, progress_callback):
         """Run RD analysis for MTBC species"""
 
-
-        '''msg = 'This tool is for Region of Difference analysis of MTBC isolates only. '\
-            'arse.'
-        reply = QMessageBox.question(self, 'Continue?', msg,
-                                        QMessageBox.Yes | QMessageBox.No, QMessageBox.No)
-        if reply == QMessageBox.No:
-            return'''
-
         self.running == True
         self.opts.applyOptions()
         kwds = self.opts.kwds
         from . import rdiff
         rdiff.create_rd_index()
-        df = self.fastq_table.model.df
-        rows = self.fastq_table.getSelectedRows()
-        data = df.iloc[rows]
+
+        data = self.get_selected()
+        if data is None or len(data) == 0:
+            return
         out = os.path.join(self.outputdir,'rd_analysis')
-        res = rdiff.find_regions(data, out, threads=kwds['threads'],
-                                 callback=progress_callback.emit)
-        self.rd_result = res
+        res = rdiff.run_samples(data, out, threads=kwds['threads'])
+
+        X = rdiff.get_matrix(res, cutoff=0.15)
+        X['species'] = X.apply(rdiff.apply_rules,1)
+        self.rd_result = X
         return
 
     def rd_analysis_completed(self):
-        """Alignment/calling completed"""
+        """RD analysis completed"""
 
-        self.info.append("finished")
+        print("finished")
         self.progressbar.setRange(0,1)
-        res = self.rd_result
-        from . import rdiff
-        X = rdiff.get_matrix(res, cutoff=0.15)
-        X['species'] = X.apply(rdiff.apply_rules,1)
+        if not hasattr(self, 'rd_result'):
+            return
+        X = self.rd_result
+        df = self.fastq_table.model.df
+
+        right = X[['species']]
+        #df = df.merge(X[['species']],left_on='sample',right_index=True,how='left')
+        df = df.set_index('sample').join(right,rsuffix='_x')
+        if 'species_x' in df.columns:
+            df['species'] = df.species.fillna(df.species_x)
+            df = df.drop('species_x', axis=1)
+        df.reset_index(inplace=True)
+        self.fastq_table.model.df = df
+        self.fastq_table.refresh()
+        #add plot
         fig,ax = plt.subplots(1,1)
         plotting.heatmap(X.set_index('species',append=True), cmap='cubehelix',ax=ax)
-
-        table = tables.DefaultTable(self.tabs, app=self, dataframe=res)
-        self.tabs.addTab(table, 'RD analysis')
-        #add plot
         w = widgets.PlotViewer(self)
         w.show_figure(fig)
         i = self.tabs.addTab(w, 'RD')
-
         self.running = False
         return
 
+    def get_selected(self):
+        """Get selected rows of fastq table"""
+
+        df = self.fastq_table.model.df
+        rows = self.fastq_table.getSelectedRows()
+        if len(rows) == 0:
+            print ('no samples selected')
+            return
+        data = df.iloc[rows]
+        return data
+
     def zoom_in(self):
 
         w = self.fastq_table
         w.zoomIn()
+        self.info.zoomIn()
         return
 
     def zoom_out(self):
 
         w = self.fastq_table
         w.zoomOut()
+        self.info.zoomOut()
         return
 
-    def show_info(self, msg):
+    def show_info(self, msg, color=None):
 
-        self.info.append(msg)
+        if color != None:
+            #self.info.appendHtml("<p style=\"color:red\">" + msg + "</p>")
+            self.info.append("<font color=%s>%s</font>" %(color,msg))
+        else:
+            self.info.append(msg)
         self.info.verticalScrollBar().setValue(
             self.info.verticalScrollBar().maximum())
         return
@@ -1263,7 +1389,7 @@ class App(QMainWindow):
         """Open the online documentation"""
 
         #import webbrowser
-        link='https://github.com/dmnfarrell/btbgenie'
+        link='https://github.com/dmnfarrell/snipgenie'
         #webbrowser.open(link,autoraise=1)
         from PySide2.QtWebEngineWidgets import QWebEngineView
         browser = QWebEngineView()
@@ -1344,6 +1470,27 @@ class WorkerSignals(QtCore.QObject):
     result = QtCore.Signal(object)
     progress = QtCore.Signal(str)
 
+class StdoutRedirect(QObject):
+    printOccur = Signal(str, str, name="print")
+
+    def __init__(self, *param):
+        QObject.__init__(self, None)
+        self.daemon = True
+        self.sysstdout = sys.stdout.write
+        self.sysstderr = sys.stderr.write
+
+    def stop(self):
+        sys.stdout.write = self.sysstdout
+        sys.stderr.write = self.sysstderr
+
+    def start(self):
+        sys.stdout.write = self.write
+        sys.stderr.write = lambda msg : self.write(msg, color="red")
+
+    def write(self, s, color="black"):
+        sys.stdout.flush()
+        self.printOccur.emit(s, color)
+
 class AppOptions(widgets.BaseOptions):
     """Class to provide a dialog for global plot options"""
 
@@ -1352,28 +1499,31 @@ class AppOptions(widgets.BaseOptions):
         self.parent = parent
         self.kwds = {}
         genomes = []
-        aligners = ['bwa','subread']
+        aligners = ['bwa','subread','minimap2']
+        platforms = ['illumina','ont']
         separators = ['_','-','|',';','~']
-        cpus = [str(i) for i in range(1,os.cpu_count()+1)]
+        cpus = os.cpu_count()
         self.groups = {'general':['threads','labelsep','overwrite'],
                         'trimming':['quality'],
-                        'aligners':['aligner'],
-                        'variant calling':['filters'],
-                        'blast':['db','identity','coverage']
+                        'aligners':['aligner','platform'],
+                        'variant calling':['filters','proximity'],
+                        #'blast':['db','identity','coverage']
                        }
-        self.opts = {'threads':{'type':'combobox','default':4,'items':cpus},
+        self.opts = {'threads':{'type':'spinbox','default':4,'range':(1,cpus)},
                     'overwrite':{'type':'checkbox','default':False},
                     'labelsep':{'type':'combobox','default':'_',
                     'items':separators,'label':'label sep','editable':True},
-
                     'aligner':{'type':'combobox','default':'bwa',
                     'items':aligners,'label':'aligner'},
-                    'db':{'type':'combobox','default':'card',
-                    'items':[],'label':'database'},
+                    'platform':{'type':'combobox','default':'illumina',
+                    'items':platforms,'label':'platform'},
                     'filters':{'type':'entry','default':app.default_filter},
-                    'identity':{'type':'entry','default':90},
-                    'coverage':{'type':'entry','default':50},
+                    'proximity': {'type':'checkbox','default':0},
                     'quality':{'type':'spinbox','default':30}
+                    #'db':{'type':'combobox','default':'card',
+                    #'items':[],'label':'database'},
+                    #'identity':{'type':'entry','default':90},
+                    #'coverage':{'type':'entry','default':50},
                     }
         return
 
