@@ -835,19 +835,18 @@ def gff_bcftools_format(in_file, out_file):
         GFF.write([new], out_handle)
         return
 
-def get_spoligotype(filename, reads_limit=500000, threshold=2):
+def get_spoligotype(filename, reads_limit=3000000, threshold=2, threads=4):
     """Get mtb spoligotype from WGS reads"""
 
-    #ref = '../snipgenie/data/dr_spacers.fa'
     ref = os.path.join(datadir, 'dr_spacers.fa')
     #convert reads to fasta
     fastq_to_fasta(filename, 'temp.fa', reads_limit)
     #make blast db from reads
     make_blast_database('temp.fa')
     #blast spacers to db
-    bl = blast_fasta('temp.fa', ref, evalue=0.1,
-                           maxseqs=100000, show_cmd=False)
-    bl=bl[(bl.qcovs>95) & (bl.mismatch<2)]
+    bl = blast_fasta('temp.fa', ref, evalue=0.1, threads=threads,
+                           maxseqs=reads_limit, show_cmd=False)
+    bl=bl[(bl.qcovs>90) & (bl.mismatch<=threshold)]
     x = bl.groupby('qseqid').agg({'pident':np.size}).reset_index()
     x = x[x.pident>=threshold]
     found = list(x.qseqid)
@@ -858,7 +857,7 @@ def get_spoligotype(filename, reads_limit=500000, threshold=2):
         else:
             s.append('0')
     s =''.join(s)
-    print (s)
+    #print (s)
     return s
 
 def get_sb_number(binary_str):
