@@ -206,12 +206,13 @@ def setWidgetValues(widgets, values):
             elif type(w) is QPlainTextEdit:
                 w.insertPlainText(str(val))
             elif type(w) is QComboBox or type(w) is QFontComboBox:
-                w.setCurrentIndex(1)
+                index = w.findText(val)
+                w.setCurrentIndex(index)
             elif type(w) is QCheckBox:
                 w.setChecked(val)
             elif type(w) is QSlider:
                 w.setValue(val)
-            elif type(w) is QSpinBox:
+            elif type(w) in [QSpinBox,QDoubleSpinBox]:
                 w.setValue(val)
     return
 
@@ -502,6 +503,12 @@ class BaseOptions(object):
         self.applyOptions()
         return
 
+    def updateWidgets(self, kwds):
+
+        for k in kwds:
+            setWidgetValues(self.widgets, {k: kwds[k]})
+        return
+
     def increment(self, key, inc):
         """Increase the value of a widget"""
 
@@ -535,8 +542,13 @@ class DynamicDialog(QDialog):
         return kwds
 
 class Editor(QTextEdit):
-    def __init__(self, parent=None, **kwargs):
+    def __init__(self, parent=None, fontsize=12, **kwargs):
         super(Editor, self).__init__(parent, **kwargs)
+        font = QFont("Monospace")
+        font.setPointSize(fontsize)
+        font.setStyleHint(QFont.TypeWriter)
+        self.setFont(font)
+        return
 
     def zoom(self, delta):
         if delta < 0:
@@ -561,6 +573,12 @@ class Editor(QTextEdit):
         elif action == zoomoutAction:
             self.zoom(-1)
 
+    def insert(self, txt):
+
+        self.insertPlainText(txt)
+        self.verticalScrollBar().setValue(self.verticalScrollBar().maximum())
+        return
+
 class PlainTextEditor(QPlainTextEdit):
     def __init__(self, parent=None, **kwargs):
         super(PlainTextEditor, self).__init__(parent, **kwargs)
